# This file is part sale_shop module for Tryton.
# The COPYRIGHT file at the top level of this repository contains
# the full copyright notices and license terms.
from trytond.model import ModelView, ModelSQL, fields
from trytond.pool import Pool

TYPE_VEHICLE = [
    ('', ''),
    ('motorcycle', 'Motorcycle'),
    ('bicycle', 'Bicycle'),
    ('Car', 'Car'),
]


class SaleShopTable(ModelSQL, ModelView):
    'Sale Shop Table'
    __name__ = 'sale.shop.table'
    shop = fields.Many2One('sale.shop', 'Shop', required=True)
    name = fields.Char('Table Name', required=True, select=True)
    active = fields.Boolean('Active', select=True)
    capacity = fields.Integer('Capacity', required=True, select=True)
    sale = fields.Many2One('sale.sale', 'Sale')
    state = fields.Selection([
        ('available', 'Available'),
        ('occupied', 'Occupied'),
        ('reserved', 'Reserved')
    ], 'State', states={'readonly': False}
    )

    @staticmethod
    def default_active():
        return True

    @staticmethod
    def default_state():
        return 'available'


class Reservation(ModelSQL, ModelView):
    'Sale Reservation'
    __name__ = 'sale.reservation'


class PartyConsumer(ModelSQL, ModelView):
    'Party Consumer'
    __name__ = 'party.consumer'
    name = fields.Char('Name', required=True)
    phone = fields.Char('Phone', required=True)
    address = fields.Char('Address', required=True)
    id_number = fields.Char('Id Number')
    birthday = fields.Date('Birthday')
    delivery = fields.Char('Delivery')
    notes = fields.Text('Notes')

    def get_rec_name(self, name=None):
        return self.name + ' | ' + self.phone + ' | ' + self.address
